#!/usr/bin/env python
# -*- coding: utf-8 -*-
from functools import partial
from os import symlink
from pathlib import Path
from sys import exit
from typing import Optional


PARENT = Path(__file__).parent
MAX_TRIES = 3


def _timewarrior_location() -> Optional[Path]:
    timew_dir = Path.home() / ".timewarrior"
    for i in range(MAX_TRIES):
        if timew_dir.is_dir():
            return timew_dir / "extensions/"
        print(f"Directory {timew_dir} not found.")
        timew_dir = Path(input("Full path to timewarrior directory: "))
    print("Maximum number of tries reached.")
    return None


def _install(file_name: str) -> None:
    src = PARENT / file_name
    if not src.is_file():
        raise Exception("Trying to install nonexisting file")
    if (timew_dir := _timewarrior_location()) is not None:
        dest = timew_dir / file_name
        try:
            symlink(str(src.resolve()), str(dest.resolve()))
            print(f"Successfully installed extension {file_name}")
        except FileExistsError:
            print(f"File {file_name} already exits in {dest}.")
    else:
        exit(1)


install_percentage = partial(_install, "percentage.py")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Percentage - Timewarrior report with %
Author: Viliam Valent

Usage:
timew percentage

OR

timew percentage :lastweek/:week/:day
"""

import json
import sys

from collections import defaultdict
from datetime import datetime
from itertools import chain
from typing import DefaultDict, Iterable, TextIO, Tuple


ParsedData = DefaultDict[str, int]
Row = Tuple[str, ...]
Lengths = Tuple[int, ...]


TAG_HEADER = "Tag"
DURATION_HEADER = "Duration"
PERCENTAGE_HEADER = "Portion"
SEPARATOR_CHAR = "-"
COLUMN_SPACING = "  "
SORTED = True
REVERSE = True
SORT_BY = 1  # valid values: 0, 1, or 2


def format_seconds(seconds: int) -> str:
    hours, rest = divmod(seconds, 3600)
    return f"{hours}:{(rest // 60):02d}"


def format_percentage(value: float) -> str:
    return f"{value:6,.1%}"


def row_str(row: Row, lengths: Lengths) -> str:
    return COLUMN_SPACING.join(f"{value:{length}}" for value, length in zip(row, lengths))


def separator_str(lengths: Lengths) -> str:
    return row_str(tuple(SEPARATOR_CHAR * length for length in lengths), lengths)


def ghetto_tabulate(totals: ParsedData) -> None:
    if not totals:
        error_and_exit("No data in selected interval.")
    total = sum(x for x in totals.values())
    header = (TAG_HEADER, DURATION_HEADER, PERCENTAGE_HEADER)
    items = (
        totals.items()
        if not SORTED
        else sorted(totals.items(), reverse=REVERSE, key=lambda row: row[SORT_BY])
    )
    body = [(tag, format_seconds(seconds), format_percentage(seconds / total)) for tag, seconds in items]
    footer = ("Total", format_seconds(total), format_percentage(1))
    lengths = tuple(max(len(row[i]) for row in chain([header, footer], body)) for i in range(3))
    output = "\n".join(
        [row_str(header, lengths), separator_str(lengths)]
        + [row_str(row, lengths) for row in body]
        + [separator_str(lengths), row_str(footer, lengths)]
    )
    print(output)


def error_and_exit(error: str) -> None:
    print(error)
    sys.exit(1)


def parse(input_stream: TextIO) -> ParsedData:
    DATEFORMAT = "%Y%m%dT%H%M%SZ"
    header = True
    config = {}
    body = ""
    for line in input_stream:
        if header:
            if line == "\n":
                header = False
                continue
            try:
                key, value = line.strip().split(": ", 1)
                config[key] = value
            except ValueError:
                continue
        else:
            body += line
    if "temp.report.tags" in config:
        error_and_exit("This report only works without tags.")
    totals: ParsedData = defaultdict(lambda: 0)
    tracked = json.loads(body)
    for session in tracked:
        start = datetime.strptime(session["start"], DATEFORMAT)
        if "end" in session:
            end = datetime.strptime(session["end"], DATEFORMAT)
        else:
            end = datetime.utcnow()
        duration = int((end - start).total_seconds())
        if not session.get("tags"):
            totals["untagged"] += duration
        else:
            if len(session["tags"]) > 1:
                error_and_exit("This report can only report one tag per session.")
            totals[session["tags"][0]] += duration
    return totals


if __name__ == "__main__":
    ghetto_tabulate(parse(sys.stdin))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['timewarrior_extensions']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['timewarrior_percentage_install = '
                     'timewarrior_extensions.install:install_percentage']}

setup_kwargs = {
    'name': 'timewarrior-extensions',
    'version': '0.1.1',
    'description': 'Extensions for Timewarrior',
    'long_description': '# timewarrior-extensions\nExtensions for [Timewarrior](https://timewarrior.net/)\n\n## Installation\nInstall the extensions\n\n```sh\n  pip install --user timewarrior-extensions\n```\nAnd install extensions you want\n\n## Extensions\n### Percentage\nTime report with %\n\n#### Installation\n```\ntimewarrior_percentage_install\n```\n\n#### Usage\n```sh\n  timew percentage\n  timew percentage :day\n  timew percentage :week\n  timew percentage :lastweek\n  timew percentage :month\n```\n\n#### Output\n```\nTag             Duration  Portion\n--------------  --------  -------\nMulti word tag  3:00       44.9%\ndevelopment     1:31       22.8%\nmeeting         1:00       15.0%\nreview          1:00       15.0%\nS2              0:08        2.1%\nS               0:01        0.2%\n--------------  --------  -------\nTotal           6:40      100.0%\n```\n\n## Development\n\n```sh\n  git clone https://github.com/ViliamV/timewarrior-extensions.git\n  cd timewarrior-extensions\n  poetry install\n  ln -rs pre-commit .git/hooks/\n```\n',
    'author': 'Viliam Valent',
    'author_email': 'viliam@valent.email',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ViliamV/timewarrior-extensions',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)
